package net.sf.corn.httpclient;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Html Form implementation for HTTP GET/POST requests. 
 * This class simply extends {@link HttpClient} class and offers some additional methods to put form fields and do get/post requests
 *  * <p><code>
 * HttpForm form= new HttpClient(new URI("https://www.googleapis.com/books/v1/volumes"); </br>
 * //optional proxy creds</br>
 * form.setProxyCredentials(yourProxyUser,yourProxyPassword);</br>
 * //optional request creds</br>
 * form.setProxyCredentials(yourUser,yourPassword);</br>
 * form.putFieldValue("q","quilting");</br>
 * HttpResponse response = form.doGet();</br>
 * if(response.hasError()){</br>
 * &nbsp System.out.println("Error:"+response.getErrorMessage()+","+response.getErrorData());</br>
 * }else{</br>
 * &nbsp System.out.println("Response Code:"+response.getResponseCode());</br>
 * &nbsp System.out.println("Response Data:"+response.getResponseData());</br>
 * &nbsp System.out.println("Cookies:"+response.getCookies());</br>
 * &nbsp System.out.println("Headers:"+response.getHeaderFields());</br>
 * }</br>
 * </code></p>
 * @author Serhat Dirik
 * @see TSAuthenticator
 * @see HttpClient
 */
public class HttpForm extends HttpClient{

	private Map<String, String> fields = new HashMap<String, String>();
	private List<String> 	    fieldsOrder = new ArrayList<String>();
	
	/**
	 * Default constructor
	 */
	public HttpForm(URI uri) {
		super(uri);
	}

	/**
	 * Clears all field values that are set previously
	 */
	public void clearFieldValues() {
		fields.clear();
		fieldsOrder.clear();
	}

	/**
	 * Adds a new field and its value to the form
	 */
	public void putFieldValue(String name, String value) {
		fields.put(name, value);
		fieldsOrder.add(name);
	}

	/**
	 * Gives all form fields as get query string
	 */
	public String toGetString() {
		StringBuilder sb = new StringBuilder();
		boolean firstField = true;
		for (String name : fieldsOrder) {
 			try {
				if (firstField) {
					firstField = false;
				} else {
					sb.append("&");
				}
				 sb.append(URLEncoder.encode(name, getCharset()));
				sb.append("=");
				if (fields.get(name) != null
						&& fields.get(name).trim().length() > 0) {
					 sb.append(URLEncoder.encode(fields.get(name),getCharset()));
				}

 			} catch (UnsupportedEncodingException e) {
 			}
		}

		return sb.toString();
	}
	/**
	 * Gives all form fields as post query string
	 */
	public String toPostString() {
		return toGetString();
	}
	/**
	 * Makes an HTTP GET request to given URI with parameters
	 * @return http response
	 * @throws IOException
	 */
	public HttpResponse doGet() throws IOException{
		return sendData(HTTP_METHOD.GET, toGetString());
	}
	/**
	 * Makes an HTTP POST request to given URI with parameters
	 * @return http response
	 * @throws IOException
	 */
	public HttpResponse doPost() throws IOException{
		return sendData(HTTP_METHOD.POST, toPostString());
	}
	
	@Override
	public String toString() {
		return new StringBuilder().append("URI:").append(getURI()).append(", Form:").append(fields.toString()).toString();
	}
	
}
